/**
 * @(#)City.java  1.0  6 November 2006
 *
 * @author Viera K. Proulx
 */

import edu.neu.ccs.gui.*;	  // to get the BufferedPanel
import edu.neu.ccs.*;	      // to get the XInt
import java.util.*;        // to get the StringTokenizer
import java.text.*;        // to get the ParseException

/** 
 * A simple class to use for learning about classes and objects. 
 * Implements the <CODE>{@link ISame ISame}</CODE> interface
 * used for tests and the <CODE>{@link Stringable Stringable}</CODE> 
 * interface used to provide input and output processing.
 */
class City implements ISame, Stringable{
  
  /*-------------------------------------------------------------------------
   Member data 
   *-----------------------------------------------------------------------*/
  
  /** The zip code for this city. 
   * A city with multiple zip codes 
   * appears in the list once for each zip code.
   */
  public int zip;
  
  /** The name of this city. */
  public String name;
  
  /** The state for this city */
  public String state;
  
  /** The longitude for this city */
  public double longitude;
  
  /** The latitude for this city */
  public double latitude;
  
  /** Decimal format to print leading zeros in zip code */
  public DecimalFormat zipFormat = new DecimalFormat("00000");
  
  /*-------------------------------------------------------------------------
   Constructor
   *-----------------------------------------------------------------------*/
  
  /** 
   * The full constructor -- zip code represented as an integer.
   *
   * @param zip the zip code for this city given as an integer.
   * @param name the name of this city.
   * @param state the state for this city.
   * @param longitude the longitude location for this city.
   * @param latitude the latitude location for this city.
   */
  public City (int zip, String name, String state, 
      double longitude, double latitude){
    this.zip   = zip;
    this.name  = name;
    this.state = state;
    this.longitude = longitude;
    this.latitude  = latitude;
  }    
  /** 
   * The copy constructor.
   *
   * @param aCity the city from which to extract the data for this instance.
   */
  public City(City aCity){
    this(aCity.zip, aCity.name, aCity.state, aCity.longitude, aCity.latitude);
  }
  
  /** 
   * The full constructor -- zip code represented 
   * as a <CODE>{@link String String}</CODE>.
   *
   * @param zip the zip code for this city 
   * given as a <CODE>{@link String String}</CODE>.
   * @param name the name of this city.
   * @param state the state for this city.
   * @param longitude the longitude location for this city.
   * @param latitude the latitude location for this city.
   */
  public City (String zip, String name, String state, 
      double longitude, double latitude){
    this.latitude = this.decimal(zip);
    this.name  = name;
    this.state = state;
    this.longitude = longitude;
    this.latitude  = latitude;
  }
  
  /** 
   * The default constructor.
   * <p> A default constructor is necessary when using the 
   * <CODE>{@link Stringable Stringable}</CODE> interface
   * to construct a new instance. Invoking the  
   * <CODE>fromStringData</CODE> method on the default instance
   * changes the instance to represent the data encoded in the
   * properly encoded input <CODE>{@link String String}</CODE>.</p>
   */
  public City(){
    this.zip   = 48170;
    this.name  = "Plymouth";
    this.state = "MI";
    this.longitude  = 83.479946;
    this.latitude = 42.368820;
  }
  
  /*-------------------------------------------------------------------------
   Methods 
   *-----------------------------------------------------------------------*/
  
  /** 
   * Determine whether this city is in the given state.
   *
   * @param aState the given state.
   */
  boolean isInState(String aState){
    return this.state.equals(aState);
  }
  
  
  /*-------------------------------------------------------------------------
   Stringable interface
   *-----------------------------------------------------------------------*/
  /**
   * Encode the city data for this instance to a 
   * <CODE>{@link String String}</CODE> (e.g. to use for the file output). 
   */
  public String toStringData(){
    String s = "" + this.zipFormat.format(this.zip) + ", " +
    this.name  + ", " +
    this.state + ", " +
    this.longitude + ", " +
    this.latitude  + "\n";
    return s;
  }
  
  /**
   * Initialize the city data for this instance from a propoerly formatted
   * <CODE>{@link String String}</CODE> (e.g. from console or GUI input). 
   */
  public void fromStringData(String s) throws java.text.ParseException{
    
    try{
      /** set up the string tokenizer to extract city data */
      StringTokenizer st = new StringTokenizer(s, ",");
      
      this.zip       = decimal(st.nextToken());
      this.name      = st.nextToken();
      this.state     = st.nextToken();
      this.longitude = doubleValue(st.nextToken());
      this.latitude  = doubleValue(st.nextToken());
    }
    catch(NoSuchElementException e){
      System.out.println(e.getMessage());
      throw new java.text.ParseException("Tokenizer Error", 0);
    }
  }
  
  /**
   * Helper: convert a <CODE>{@link String String}</CODE> 
   * to an integer 
   * 
   * @param s the input <CODE>{@link String String}</CODE>.
   * @throws to report parsing errors.
   */
  protected int intValue(String s) throws ParseException{
    
    return (new XInt(s)).intValue();
  }    
  
  /**
   * Helper: convert a <CODE>{@link String String}</CODE> 
   * to a double 
   * 
   * @param s the input <CODE>{@link String String}</CODE>.
   * @throws to report parsing errors.
   */
   protected double doubleValue(String s) throws ParseException{
    return (new XDouble(s)).doubleValue();
  }

  /*-------------------------------------------------------------------------
   ISame interface
   *-----------------------------------------------------------------------*/
  /**
   * Determine whether this city is the same as the given object 
   *
   * @param obj the object to compare with.
   */
  public boolean same(Object obj){
    return
    this.zip == ((City)obj).zip &&
    this.name.equals(((City)obj).name) &&
    this.state.equals(((City)obj).state) &&
    this.sameDouble(this.longitude, ((City)obj).longitude, 0.000001) &&
    this.sameDouble(this.latitude, ((City)obj).latitude, 0.000001);
  }
  
  /**
   * A helper method to compare doubles 
   *
   * @param d1 the first number to compare
   * @param d2 the second number to compare
   * @param limit Numbers within the limit are considered to be the same.
   */
  protected boolean sameDouble(double d1, double d2, double limit){
    return Math.abs(d1 - d2) < limit;
  }
  
    
  /*-------------------------------------------------------------------------
   Console print
   *-----------------------------------------------------------------------*/
  /** 
   * Represents this instance as a pretty-printable 
   * <CODE>{@link String String}</CODE>.
   * 
   */
  public String toString(){
    return ("new " + getClass() + "(" + 
        this.zipFormat.format(this.zip)   + ",   " + 
        this.name  + ",  " + 
        this.state + ", " + 
        this.longitude + ", " + 
        this.latitude + ")\n");
  }
  
  /**
   * Helper method to convert zip code string to integer.
   * Mask the <CODE>{@link ParseException ParseException}</CODE>
   * by returning zero value.
   * 
   * @param s the zip code represented 
   * as a <CODE>{@link String String}</CODE>.
   * @return the zip code represented as an integer.
   */
  public int decimal(String s){
    
    char[] c = s.toCharArray();
    int n = 0;
    try {
      for (int i = 0; i < 5; i++){
        n = 10 * n + intValue("" + c[i]);
      }
    }
    catch(ParseException e){
      System.out.println("Converting zip to int " + e);
    }
    return n;
  }
  
}
