/**
 * @(#)TimerTests  1.0  13 November 2006
 *
 * @author Viera K. Proulx
 */

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;

import edu.neu.ccs.console.ConsoleAware;
/**
 * <P>A class that sets up and runs a number of timing tests on any of the
 * given sorting algorithms, using the selected datasets, and selecting
 * one of three possible comparators.</P>
 * <P>Comment out the unneeded choices within the methods 
 * <CODE>makeComparators</CODE>, <CODE>makeAlgos</CODE>, and 
 *
 */
class TimerTests implements ConsoleAware{
  
  // the three comparators
  Comparator<City> byName = new ComparatorByName();
  // Comparator<City> byZip = new ComparatorByZip();
  // Comparator<City> byLatitude = new ComparatorByLatitude();
  
  // the six sorting algorithms
  // ASortAlgo selectarray = new ArrSortSelection<City>(byName);
  // ASortAlgo insertarray = new ArrSortInsertion<City>(byName);
  ASortAlgo insertlist  = new AListSortInsertion<City>(byName);
  // ASortAlgo bstsort     = new ABSTsort<City>(byName);
  // ASortAlgo quicklist   = new AListSortQuickSort<City>(byName);
  // ASortAlgo quickarray  = new ArrSortQuick<City>(byName);
  
  // an arraylist of comparators to use
  public ArrayList<Comparator> comparators = new ArrayList<Comparator>();
  public ArrayList<String> compNames = new ArrayList<String>();

  // an arraylist of algorithms to run
  public ArrayList<ASortAlgo> algorithms = new ArrayList<ASortAlgo>();

  // original data set of all cities available
  TestData testdata = new TestData();
  
  // records of the start and end time
  long startTime;         // time of the start of the test
  long endTime;           // time of the end of the test
  
  /**
   * The constructor: create an array of all possible 
   * <CODE>{@link Comparator Comparator}</CODE> choices.
   */ 
  TimerTests(){ 
    makeComparators();
  }
  
  /**
   * <P>Build an <CODE>{@link ArrayList ArrayList}</CODE> of all 
   * possible <CODE>{@link Comparator Comparator}</CODE> choices.</P>
   * <P>Comment out those you do not choose.</P>
   */ 
  public void makeComparators(){
    comparators.add(byName);
    compNames.add("By Name");
    // comparators.add(byZip);
    // compNames.add("By Zip");
    // comparators.add(byLatitude);
    // compNames.add("By Latitude");
  }
   
  /**
   * <P>Build an <CODE>{@link ArrayList ArrayList}</CODE> of all 
   * possible algorithm choices for the given 
   * <CODE>{@link Comparator Comparator}</CODE>.</P>
   * <P>Comment out those you do not choose.</P>
   */ 
  public void makeAlgos(Comparator<City> comp){
    // algorithms.add(new ArrSortSelection<City>(comp));
    // algorithms.add(new ArrSortInsertion<City>(comp));
    algorithms.add(new AListSortInsertion<City>(comp));
    // algorithms.add(new ABSTsort<City>(comp));
    // algorithms.add(new AListSortQuickSort<City>(comp));
    // algorithms.add(new ArrSortQuick<City>(comp));
  } 
  
  /**
   * <P>Build an <CODE>{@link ArrayList ArrayList}</CODE> of all 
   * possible algorithm choices for each of the given 
   * <CODE>{@link Comparator Comparator}</CODE> choices.</P>
   * <P>Comment out those you do not choose.</P>
   */ 
  public void makeAllAlgos(){
    makeAlgos(byName);
    // makeAlgos(byZip);
    // makeAlgos(byLatitude);
  }
  
  /**
   * Run all selected tests and report the results as an 
   * <CODE>{@link ArrayList ArrayList}</CODE> of 
   * CODE>{@link Result Result}</CODE> data.
   * 
   * @return the <CODE>{@link ArrayList ArrayList}</CODE> of 
   * CODE>{@link Result Result}</CODE> data
   */
  public ArrayList<Result> allTests(){
    ArrayList<Result> results = new ArrayList<Result>();
    
    // make all algorithm-comparator pairs
    makeAllAlgos();
    
    // for each data set and each algorithm/comparator pair run the tests 
    
    // select the datasets to use in this run
    // dindex ranges from 0 to 14 for the 15 possible datasets
    for (int dindex = 2; dindex < 4; dindex = dindex + 1){
      
      // run the timing test for every algorithm in the 'algorithms' ArrayList
      for (int aindex = 0; aindex < algorithms.size(); aindex = aindex + 1)  
        results.add(
            runATest(algorithms.get(aindex), testdata.datasets.get(dindex))); 
    }
    
    System.out.println("Number of comparators: " + comparators.size());
    System.out.println("Number of algorithms:  " + algorithms.size());
    System.out.println("" + results.size() + " tests completed:");
    for (int i = 0; i < results.size(); i = i + 1)
      System.out.println(results.get(i).toString());
    return results;
  }
    
  /**
   * Run one test of a given sorting algorithm using its comparator
   * and the given dataset.
   * 
   * @param algo the algorithm, initialized with a 
   * <CODE>{@link Comparator comparator}</CODE>
   * @param dataset to be sorted
   * @return the time it took to perform the sorting
   */
  public long runOneTest(ASortAlgo<City> algo, DataSet dataset) {
    
    // select data to be sorted and record the information
    Traversal<City> source = dataset.getTraversal();
    int size = dataset.size;
    String type = dataset.type;
    
    // variable to record the result
    Traversal<City> result;
    
    // initialize the data for the algorithm
    algo.initData(source);
    
    // run the timed test and return the measured time
    this.startTime = (new Date()).getTime();
    result = algo.sort();
    this.endTime = (new Date()).getTime();
    return this.endTime - this.startTime;
  }
  
  /**
   * Run one test of a given sorting algorithm using its comparator
   * and the given dataset. Report the results as a 
   * <CODE>{@link Result Result}</CODE> object.
   * 
   * @param algo the algorithm, initialized with a 
   * <CODE>{@link Comparator comparator}</CODE>
   * @param dataset to be sorted
   * @return a <CODE>{@link Result Result}</CODE> object that records
   * the time, the name of the algorithm, the name of the comparator, 
   * the size and the type of the data.
   */
  public Result runATest(ASortAlgo<City> algo, DataSet dataset) {
    
    // select data to be sorted and record the information
    int size = dataset.size;
    String type = dataset.type;
    
    // variable to record the result
    Traversal<City> result;
    
    // initialize the data for the algorithm
    algo.initData(dataset.getTraversal());
    
    // run the timed test and return the measured time
    this.startTime = (new Date()).getTime();
    result = algo.sort();
    this.endTime = (new Date()).getTime();
    
    if (isSorted(result, algo.comp))
    return new Result(size, type, algo.algoName, 
        algo.comp.getClass().getName(),
        this.endTime - this.startTime);  
    else
      return new Result(size, type, algo.algoName, 
          algo.comp.getClass().getName(),
          -999);
      //throw new IllegalStateException("The result is not sorted");
  }
  
  /**
   * Is the data generated by the given traversal sorted by comparator?
   */
  public boolean isSorted(Traversal<City> tr, Comparator<City> comp){
    // TO BE DONE ...........
      return true;
  }  
  
  /**
   * A sample test for this class
   */ 
  public void sampleTest(){
    
    // example of running the test for one choice of
    // algorithm, data to sort, and comparator
    long runtime = this.runOneTest(
        new AListSortInsertion<City>(this.byName), testdata.datasets.get(0));
    
    // we should print more info here - size of the data set,
    // is it random or sequential
    console.out.println("Selection sort ArrayList time: " + 
        runtime + "\n");
    
    console.out.println((this.runATest(
        new AListSortInsertion<City>(this.byName), testdata.datasets.get(2))).toString());
  }
  
  /**
   * <P>The main method for the project.</P>
   * <P>Runs all selected tests.</P>
   * @param argv unused
   */
  public static void main(String[] argv){
    TimerTests tt = new TimerTests();
    // tt.sampleTest();
    
    ArrayList<Result> results = tt.allTests();
    System.out.println("Completed " + results.size() + " tests.");
    
    City northville = new City(48167, "Northville", "MI", 83.479355, 42.426245);
    City plymouth = new City(48170, "Plymouth", "MI", 83.479946, 42.36882);
    City gibraltar = new City(48173, "Gibraltar", "MI", 83.216196, 42.07918);
    City romulus = new City(48174, "Romulus", "MI", 83.358288, 42.220304);
    City saline = new City(48176, "Saline", "MI", 83.784936, 42.169844);
    City duluth1 = new City(55806, "Duluth", "MN", 92.127871, 46.771457);
    City duluth2 = new City(55807, "Duluth", "MN", 92.169821, 46.740783);
     
    AList<City> empty = new MTList<City>();
    
    AList<City> longunsorted = new ConsList<City>(northville,
        new ConsList<City>(plymouth,
            new ConsList<City>(romulus,
                new ConsList<City>(duluth1,
                    new ConsList<City>(gibraltar,
                        new ConsList<City>(saline,
                            new ConsList<City>(duluth2, empty)))))));

   
    AList<City> longsorted = new ConsList<City>(duluth1,
        new ConsList<City>(duluth2,
            new ConsList<City>(gibraltar,
                new ConsList<City>(northville,
                    new ConsList<City>(plymouth,
                        new ConsList<City>(romulus,
                            new ConsList<City>(saline, empty)))))));
    System.out.println("sorted test - success:" + 
        tt.isSorted(longsorted,
                    new ComparatorByName()));

    System.out.println("sorted test - failure:" + 
        tt.isSorted(longunsorted,
                    new ComparatorByName()));

  }
}